import logging
import json
import re
from scrapy import signals
from scrapy.settings import BaseSettings

logger = logging.getLogger(__name__)


class SpiderSettingsLogging:
    @classmethod
    def from_crawler(cls, crawler):
        ext = cls()
        crawler.signals.connect(ext.spider_closed, signal=signals.spider_closed)
        return ext

    def spider_closed(self, spider):
        settings = spider.settings
        if not settings.getbool("SETTINGS_LOGGING_ENABLED"):
            return

        regex = settings.get("SETTINGS_LOGGING_REGEX")
        indent = settings.get("SETTINGS_LOGGING_INDENT")
        if regex is not None:
            settings = {k: v for k, v in settings.items() if re.search(regex, k)}

        # nested settings will be BaseSettings objects that are not JSON seriable
        settings = {
            k: dict(v) if type(v) is BaseSettings else v for k, v in settings.items()
        }

        logger.debug(json.dumps(settings, indent=indent))



from scrapy import Spider
from scrapy.crawler import Crawler
from scrapy_settings_log import SpiderSettingsLogging
import logging

LOGGER = logging.getLogger(__name__)


def MockSpider(settings):
    crawler = Crawler(Spider, settings=settings)
    return Spider.from_crawler(crawler, name="dummy")


def test_disabled(caplog):
    spider = MockSpider({})
    logger = SpiderSettingsLogging()
    logger.spider_closed(spider)

    with caplog.at_level(logging.DEBUG):
        logger.spider_closed(spider)

    assert not caplog.text


def test_log_all(caplog):
    settings = {
        "SETTINGS_LOGGING_ENABLED": True,
    }

    spider = MockSpider(settings)
    logger = SpiderSettingsLogging()
    with caplog.at_level(logging.DEBUG):
        logger.spider_closed(spider)

    # won't check specifics here as the default settings
    # can vary with scrapy versions - presence is enough
    assert caplog.text


def test_log_filtered(caplog):
    settings = {
        "SETTINGS_LOGGING_ENABLED": True,
        "DUMMY_INT": 4,
        "DUMMY_STR": "foo",
        "SETTINGS_LOGGING_REGEX": "DUMMY",
    }

    spider = MockSpider(settings)
    logger = SpiderSettingsLogging()
    with caplog.at_level(logging.DEBUG):
        logger.spider_closed(spider)

    assert '{"DUMMY_INT": 4, "DUMMY_STR": "foo"}' in caplog.text


def test_log_indented(caplog):
    settings = {
        "SETTINGS_LOGGING_ENABLED": True,
        "DUMMY_INT": 4,
        "DUMMY_STR": "foo",
        "SETTINGS_LOGGING_REGEX": "DUMMY",
        "SETTINGS_LOGGING_INDENT": 4,
    }

    spider = MockSpider(settings)
    logger = SpiderSettingsLogging()
    with caplog.at_level(logging.DEBUG):
        logger.spider_closed(spider)

    assert '{\n    "DUMMY_INT": 4,\n    "DUMMY_STR": "foo"\n}' in caplog.text



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="scrapy-settings-log",
    version="1.0.2",
    author="Roy Healy",
    author_email="roy.healy87@gmail.com",
    description="An extension that allows a user to display all or some of their scrapy spider settings at runtime.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/further-reading/scrapy-settings-log",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'scrapy',
      ],
    python_requires='>=3.6',
    include_package_data=True,
)
